from mido import MidiFile, MidiTrack, Message, MetaMessage, bpm2tempo
import math
import matplotlib.pyplot as plt

# 1. Get Pi digits (e.g., use math.pi)
pi_digits = [int(d) for d in str(math.pi)[2:152]]

# 2. Create MIDI file and track
mid = MidiFile()
track = MidiTrack()
mid.tracks.append(track)

# 3. Program Change (Acoustic Grand Piano, channel 1)
track.append(Message('program_change', channel=0, program=0, time=0))

# 4. Set Tempo to 120 BPM
tempo = bpm2tempo(120)
track.append(MetaMessage('set_tempo', tempo=tempo, time=0))

# 5. Add metadata
track.append(MetaMessage('track_name', name='Pi Sonification', time=0))
track.append(MetaMessage('copyright', text='© 2025 Ankan Debnath', time=0))

# 6. Map Pi digits to notes with harmony and dynamic velocity
for i, d in enumerate(pi_digits):
    note = 60 + d
    velocity = 64 + (d % 16)  # Dynamic velocity
    track.append(Message('note_on', channel=0, note=note, velocity=velocity, time=0))
    track.append(Message('note_off', channel=0, note=note, velocity=velocity, time=120))

    # Add harmony (third and fifth intervals)
    if i % 3 == 0:  # Add harmony every third note
        track.append(Message('note_on', channel=0, note=note + 4, velocity=velocity - 10, time=0))
        track.append(Message('note_off', channel=0, note=note + 4, velocity=velocity - 10, time=120))
        track.append(Message('note_on', channel=0, note=note + 7, velocity=velocity - 10, time=0))
        track.append(Message('note_off', channel=0, note=note + 7, velocity=velocity - 10, time=120))

# 7. Save the file
mid.save('pi_sonification.mid')
print("Saved 'pi_sonification.mid'")

# 8. Visualization of Pi digits and notes
plt.figure(figsize=(12, 6))
plt.plot(pi_digits, marker='o', linestyle='-', color='b', label='Pi Digits')
plt.title('Pi Digits and Corresponding Notes')
plt.xlabel('Digit Index')
plt.ylabel('Digit Value / Note')
plt.legend()
plt.grid(True)
plt.show()
